/* ************************************************************	*/
/*			unsh.c					*/
/*								*/
/* A Quick & Dirty program to scan down a shar/uue file (such as*/
/* might be found in a news item), find the interpreter line	*/
/* (eg. #!/bin/sh ), and send the rest of the file to the	*/
/* interpreter...						*/
/*								*/
/* Usage:							*/
/*		unsh [-#] [-u] [<directory>]			*/
/*								*/
/* No options - look for "#!" line on stdin and exec specified	*/
/* 		interpreter on rest of file			*/
/*								*/
/* -#		look for first "#" line on stdin and exec to	*/
/*		"/bin/sh" - useful if archiver hasn't put in	*/
/*		an interpreter line				*/
/*								*/
/* -u		pass whole file to "uudecode" for processing	*/
/*								*/
/* <directory>	chdir to specified directory before any of the	*/
/*		above - if it doesn't exist, have a go at	*/
/*		creating it first.				*/
/* 								*/
/* All so simple it's nearly uncommented!			*/
/*								*/
/* (C) Copyright Mike Bell 1987					*/
/* Permission granted to use/modify/copy (but not sell) provided*/
/* copyright notice(s) retained and that you mark any changes as*/
/* your own.							*/
/* ************************************************************	*/
/* History:							*/
/*	Flag	Date	Wot					*/
/*	-------------------------------------------		*/
/*	MB000	4/8/87	Comments added				*/
/*	BE001	7/8/87	Function mkdir added for USG systems	*/
/*	ADDW1	9/8/87	ditto					*/
/*								*/
/* Modifiers:							*/
/*	MB	Mike Bell	mb@camcon.uCuc			*/
/*				Cambridge Consultants UK	*/
/*								*/
/*	BE	Bert van Es					*/
/*				Northern Telecom Netherlands	*/
/*								*/
/*	ADDW	Alain Williams	addw@phcomp.uucp		*/
/*				Parliament Hill Computers UK	*/
/*								*/
/* ************************************************************	*/

#include <stdio.h>
#include <errno.h>

extern int errno;

extern char *gets();

pexit( msg )				/* Print message & die */
    char *msg;
{
    perror( msg );
    exit( errno );
}

#define MAXLINE 512			/* Max input line - longer cdumps! */

main ( argc, argv )
    int argc;
    char **argv;
{
    int index;

    int uu_flag = 0;			/* Pass to uudecode flag */
    int hash_flag = 0;			/* No #! - use # flag    */

    char line[MAXLINE];
    char shell[MAXLINE];
    char arg1[MAXLINE];

    setbuf( stdin, NULL );	/* Disable buffering (screws exec) */


    if ( (argc > 1) && (strcmp( argv[1], "-#") == 0 ))
    {
	hash_flag = 1;
	argv++; argc--;
    }

    if ( (argc > 1) && (strcmp( argv[1], "-u") == 0 ))
    {
	uu_flag = 1;
	argv++; argc--;
    }


    if (uu_flag && hash_flag)
    {
	printf( "unsh - Incompatible options\n" );
	exit( 1 );
    }

    if (argc <= 1 )
    	goto nodir;			/* Yuk - but keeps code simple */
	
    if ( chdir( argv[1] ) == 0 )
	goto ok;			/* Yeagghh! */

    if ((errno == ENOENT)		/* No such directory? */
    &&  (mkdir(argv[1], 0777) == 0) )	/* Default access permission */
    {
	printf( "unsh - Created directory: %s\n", argv[1] );
	
    	if  (chdir(argv[1]) == 0) 
	    goto ok;			/* Last one!! */
    }

    pexit( "unsh - Could not chdir" );

ok:
    printf( "unsh - Changed to directory: %s\n", argv[1] );

nodir:



    if (uu_flag)
    {
	printf( "unsh - Sending to: uudecode\n" );
	execlp( "uudecode", "uudecode", NULL );
	pexit( "unsh - execl failed" );
    }

    while (gets(line) != NULL)
    {
	if (strncmp( line, "#!", 2) == 0 )	/* Interpreter line? */
	{
	    for (index = 2; (line[index] == ' ') && (index <MAXLINE); index++ )
		;
	    strcpy( shell, &line[index] );
	    printf( "unsh - Sending to: %s\n", shell );
		
	    strcpy( arg1, "-" );
	    strcat( arg1, shell );

	    execlp( shell, arg1, NULL );    /* Normally "/bin/sh" "-/bin/sh" */
	    pexit( "unsh - execl failed" );
	}

	if (hash_flag && (strncmp( line, "#", 1) == 0))
	{
	    printf( "unsh - Sending to: %s\n", "/bin/sh" );
	    execlp( "/bin/sh", "-/bin/sh", NULL );
	    pexit( "unsh - execl failed" );
	}
    }
    printf( "unsh - Nothing found\n" );	
    exit( 1 );

}

#ifdef USG
/* ************************************************************	*/
/* Function mkdir - "missing" on USG systems			*/
/* BE001/ADDW1 	both contributed similar versions of this	*/
/* ************************************************************	*/

#include <signal.h>
/*
 * make a directory. 
 */
mkdir(path, perm)
char *path;
int perm;
{
	int pid, status;
	char *p;


	if (pid=fork()) {
		status = fwait(pid);
		if (pid=fork())
			(void) fwait(pid);
		else {
			(void) chmod(path, perm);
			_exit(0);
		}
	} else {
		(void) execlp("mkdir", "mkdir", path, (char *)NULL);
	    	pexit( "unsh - mkdir failed" );
	}
	return status;
}


/*
 * Wait on a child process.
 */
fwait(pid)
register int pid;
{
	register int w;
	int status;
	int (*onhup)(), (*onint)();

	onint = signal(SIGINT, SIG_IGN);
	onhup = signal(SIGHUP, SIG_IGN);
	while ((w = wait(&status)) != pid && w != -1)
		;
	if (w == -1)
		status = -1;
	(void) signal(SIGINT, onint);
	(void) signal(SIGHUP, onhup);
	return status;
}

#endif
